﻿using e_teaching.Model;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage.Streams;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

// La plantilla de elemento Página en blanco está documentada en http://go.microsoft.com/fwlink/?LinkId=234238

namespace e_teaching
{
    /// <summary>
    /// Una página vacía que se puede usar de forma independiente o a la que se puede navegar dentro de un objeto Frame.
    /// </summary>
    public sealed partial class AddTask : Page
    {
        ObservableCollection<Profile> ListaAlumnosSourceCollection;
        ObservableCollection<Profile> ListaAlumnosNoRelacionadosSourceCollection;
        ObservableCollection<Message> ListaMensajesSourceCollection;
        Profile profileLogged = new Profile();
        ResourceLoader loader = new ResourceLoader();

        public AddTask()
        {
            this.InitializeComponent();
            GetInfo();           
        }       

        private async void GetInfo()
        {
            //comprobar si perfil tiene info
            var prof = await App.MobileService.GetTable<Profile>().Where(p => p.ID == MainPage.UserLogged.ID).ToListAsync();
            if (prof.Any())
            {
                FillProfileInfo(prof.FirstOrDefault());

                //cargar alumnos
                await FillStudentsList();
            }
            else
            {
                this.gridInfoPerfil.Visibility = Visibility.Collapsed;
            }

        }

        private void FillProfileInfo(Profile prof)
        {
            var profile = prof;
            profileLogged = prof;

            this.gridInfoPerfil.Visibility = Visibility.Visible;

            this.txtNombre.Text = profile.Name;
            this.txtApellidos.Text = profile.Surname;

            BitmapImage bitmapImage = new BitmapImage();
            bitmapImage.DecodePixelWidth = 600;
            bitmapImage = ConvertToImage(profile.Image);

            this.ImageProfile.Source = bitmapImage;
        }

        private async Task FillStudentsList()
        {
            EnableProgressRing();
            //estudiantes asociados a profesor           

            var listProfessorStudents = await App.MobileService.GetTable<ProfessorStudent>().Where(p => p.IDProfessor == MainPage.UserLogged.ID).Select(p => p.IDStudent).ToListAsync();

            if (listProfessorStudents.Any())
            {
                var studentsList = await App.MobileService.GetTable<Profile>().Where(s => listProfessorStudents.Contains(s.ID)).ToListAsync();

                this.ListaAlumnosSourceCollection = new ObservableCollection<Profile>(studentsList);

                this.listViewAlumnos.ItemsSource = ListaAlumnosSourceCollection;

                this.listViewAlumnos.Visibility = Visibility.Visible;
            }   
            else
            {
                this.listViewAlumnos.Visibility = Visibility.Collapsed;
            }

            DisableProgressRing();
        }

        private void gridSinInfo_Tapped(object sender, TappedRoutedEventArgs e)
        {
            //navegar a rellenar información de perfil
            this.Frame.Navigate(typeof(EnterProfileInfo));
        }

        public BitmapImage ConvertToImage(byte[] value)
        {
            byte[] rawImage = value;

            using (InMemoryRandomAccessStream ms = new InMemoryRandomAccessStream())
            {
                using (DataWriter writer = new DataWriter(ms.GetOutputStreamAt(0)))
                {
                    writer.WriteBytes((byte[])rawImage);

                    // The GetResults here forces to wait until the operation completes
                    // (i.e., it is executed synchronously), so this call can block the UI.
                    writer.StoreAsync().GetResults();
                }

                BitmapImage image = new BitmapImage();
                image.SetSource(ms);
                return image;
            }
        }

        private void txtEditarInfo_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(EnterProfileInfo));
        }

        private async Task<bool> CheckStudentSelected()
        {
            if (listViewAlumnos.SelectedItems.Count > 0)
            {
                return true;
            }
            else
            {
                var dialog = new MessageDialog(loader.GetString("ShouldSelectStudentTask"));
                await dialog.ShowAsync();
                return false;
            }
        }

        private async void AssignTask()
        {
            if (await CheckTaskIsNotEmpty() == true && await CheckStudentSelected() == true)
            {
                try
                {
                    EnableProgressRing();
                    foreach (Profile student in listViewAlumnos.SelectedItems)
                        {
                            Tasks task = new Tasks();
                            task.Date = DateTime.Now;
                            task.OriginId = MainPage.UserLogged.ID;
                            task.DestinationId = student.ID;
                            task.Content = this.txtTarea.Text;
                            task.Subject = string.IsNullOrEmpty(this.txtTipoTarea.Text) ? "Sin Tipo" : this.txtTipoTarea.Text;

                            //asignar tarea
                            await App.MobileService.GetTable<Tasks>().InsertAsync(task);
                        }
                   
                    
                }catch(Exception e)
                {
                    var dialog = new MessageDialog(loader.GetString("ErrorAssigningTask"));
                    await dialog.ShowAsync();
                }

                DisableProgressRing();

                var dialog2 = new MessageDialog(loader.GetString("TaskAssigned"));
                await dialog2.ShowAsync();

                //limpiar formulario
                this.txtTipoTarea.Text = string.Empty;
                this.txtTarea.Text = string.Empty;
            }

        }

        private void btnAsignarTarea_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.AssignTask();
        }

        private async Task<bool> CheckTaskIsNotEmpty()
        {
            if (string.IsNullOrEmpty(this.txtTarea.Text))
            {
                var dialog = new MessageDialog(loader.GetString("TaskIsEmpty"));
                await dialog.ShowAsync();
                return false;
            }

            return true;
        }

        private async void txtCerrarSesion_Tapped(object sender, TappedRoutedEventArgs e)
        {
           await LogOut();
        }

        private async Task LogOut()
        {
            MessageDialog showDialog = new MessageDialog(loader.GetString("SureLogout"));
            showDialog.Commands.Add(new UICommand(loader.GetString("Yes"))
            {
                Id = 0
            });
            showDialog.Commands.Add(new UICommand(loader.GetString("No"))
            {
                Id = 1
            });
            showDialog.DefaultCommandIndex = 0;
            showDialog.CancelCommandIndex = 1;
            var result = await showDialog.ShowAsync();
            if ((int)result.Id == 0)
            {
                //borrar settings de sesión
                DeleteAutoLogin();

                //navegar a menú de login
                this.Frame.Navigate(typeof(MainPage));
            }
        }

        public void DeleteAutoLogin()
        {
            MainPage.localSettings.Values.Remove("user");
            MainPage.localSettings.Values.Remove("pass");
        }
        
       
        private void EnableProgressRing()
        {
            progressRing.IsActive = true;
            progressRing.Visibility = Visibility.Visible;
        }

        private void DisableProgressRing()
        {
            progressRing.IsActive = false;
            progressRing.Visibility = Visibility.Collapsed;
        }

        private void btmHome_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(MenuProfessor));
        }

        private void btmManageAcademicRecord_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(ManageAcademicRecord));
        }
    }
}
