﻿using e_teaching.Model;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage.Streams;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

// La plantilla de elemento Página en blanco está documentada en http://go.microsoft.com/fwlink/?LinkId=234238

namespace e_teaching
{
    /// <summary>
    /// Una página vacía que se puede usar de forma independiente o a la que se puede navegar dentro de un objeto Frame.
    /// </summary>
    public sealed partial class EnterProfileInfo : Page
    {
      
        byte[] imageFile;
        ResourceLoader loader = new ResourceLoader();

        public EnterProfileInfo()
        {
            this.InitializeComponent();

        }
        
        private async void btnSave_Tapped(object sender, TappedRoutedEventArgs e)
        {
            var profile = new Profile();
            profile.ID = MainPage.UserLogged.ID;
            profile.Name = this.txtNombre.Text;
            profile.Surname = this.txtApellidos.Text;
            if (imageFile != null)
            {
                profile.Image = imageFile;
            }

            //Insertar en bbdd
            var result = await SaveDataInDB(profile);

            if (result)
            {
                //navegar a pantalla principal
                switch (MainPage.UserLogged.Role)
                {
                    case 1: //profesor
                        Frame.Navigate(typeof(MenuProfessor));
                        break;
                    case 2: //alumno
                        Frame.Navigate(typeof(MenuStudent));
                        break;
                    case 3: //padres
                        Frame.Navigate(typeof(MenuParent));
                        break;
                }
            }
        }

        private async void txtSelecFoto_Tapped(object sender, TappedRoutedEventArgs e)
        {
            var picker = new Windows.Storage.Pickers.FileOpenPicker();
            picker.ViewMode = Windows.Storage.Pickers.PickerViewMode.Thumbnail;
            picker.SuggestedStartLocation =
                Windows.Storage.Pickers.PickerLocationId.PicturesLibrary;
            picker.FileTypeFilter.Add(".jpg");
            picker.FileTypeFilter.Add(".jpeg");
            picker.FileTypeFilter.Add(".png");

            Windows.Storage.StorageFile file = await picker.PickSingleFileAsync();
            if (file != null)
            {
                using (IRandomAccessStream fileStream = await file.OpenAsync(Windows.Storage.FileAccessMode.Read))
                {
                    BitmapImage bitmapImage = new BitmapImage();
                    bitmapImage.DecodePixelWidth = 600;
                    await bitmapImage.SetSourceAsync(fileStream);
                    imageFile = await ConvertToByteArray(fileStream);
                    image.Source = bitmapImage;
                }
            }
        }

        private async Task<byte[]> ConvertToByteArray(IRandomAccessStream s)
        {
            var dr = new DataReader(s.GetInputStreamAt(0));
            var bytes = new byte[s.Size];
            await dr.LoadAsync((uint)s.Size);
            dr.ReadBytes(bytes);
            return bytes;
        }

        private async Task<bool> SaveDataInDB(Profile profile)
        {
            try
            {
                //comprobar existe usuario en bbdd
                var prof = await App.MobileService.GetTable<Profile>().Where(p => p.ID == profile.ID).ToListAsync();
                if (prof.Any())
                {
                    //actualizar
                    await App.MobileService.GetTable<Profile>().UpdateAsync(profile);
                }
                else
                {
                    //insertar 
                    await App.MobileService.GetTable<Profile>().InsertAsync(profile);
                }
                return true;
            }

            catch(Exception e)
            {
                if(e.Message.Contains("too large"))
                {
                    var dialog = new MessageDialog(loader.GetString("PictureTooLarge"));
                    await dialog.ShowAsync();                    
                }

                var dialog2 = new MessageDialog(loader.GetString("ErrorUpdateInfo"));
                await dialog2.ShowAsync();
                return false;
            }
            
        }

        private void btnCancel_Tapped(object sender, TappedRoutedEventArgs e)
        {
            Frame.GoBack();
        }
    }
}
