﻿using e_teaching.Model;
using Microsoft.WindowsAzure.MobileServices;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;


// La plantilla de elemento Página en blanco está documentada en http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace e_teaching
{
    /// <summary>
    /// Página vacía que se puede usar de forma independiente o a la que se puede navegar dentro de un objeto Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
       
        //private IMobileServiceSyncTable<TodoItem> todoTable = App.MobileService.GetSyncTable<TodoItem>(); // offline sync

        public static User UserLogged;
       public static ApplicationDataContainer localSettings = ApplicationData.Current.LocalSettings;

        //public static ProgressRing progressRing = new ProgressRing();

        ResourceLoader loader = new ResourceLoader();
        public MainPage()
        {
            this.InitializeComponent();

            CheckAutoLogin();

        }

        private void SaveAutoLogin()
        {
            localSettings.Values["user"] = this.userTxt.Text;
            localSettings.Values["pass"] = this.passTxt.Password;            
        }

        private async Task CheckAutoLogin()
        {
            string user = localSettings.Values["user"].ToString();
            string pass = localSettings.Values["pass"].ToString();

            if (user != null && pass != null)
            {
                //login
                this.userTxt.Text = user;
                this.passTxt.Password = pass;
                await DoLogin(user, pass);
            }
        }

        

        private async void LoginBtn_Tapped(object sender, TappedRoutedEventArgs e)
        {
            //si autologin checked
            if (this.checkBoxSesion.IsChecked == true)
            {
                SaveAutoLogin();
            }

            if(CheckPasswordIsNotEmpty() == true && CheckUserNameIsNotEmpty() == true)
            {
                await DoLogin(this.userTxt.Text, this.passTxt.Password);
            }
        }

        private async Task DoLogin(string user, string pass)
        {
            bool LoginOK = await Login(user, pass);
            if (LoginOK) //navegar a menú principal 
            {
                switch (UserLogged.Role)
                {
                    case 1: //profesor
                        Frame.Navigate(typeof(MenuProfessor));
                        break;
                    case 2: //alumno
                        Frame.Navigate(typeof(MenuStudent));
                        break;
                    case 3: //padres
                        Frame.Navigate(typeof(MenuParent));
                        break;
                }
            }
        }



        private async Task<bool> Login(string userName, string pass)
        {
            EnableProgressRing();

            User user = new User()
            {
                UserName = userName,
                Password = pass,
            };

            var loginOK = false;

            try
            {
                var userList = await App.MobileService.GetTable<User>().Where(u => u.UserName == userName).ToListAsync();

                if (userList.Any())
                {
                    //comprobar contraseña
                    loginOK = userList.FirstOrDefault().Password.Equals(pass) ? true : false;
                    if (!loginOK)
                    {
                        var dialog = new MessageDialog(loader.GetString("PasswordIncorrect"));
                        await dialog.ShowAsync();
                    }
                    else
                    {
                        UserLogged = userList.FirstOrDefault();
                    }
                }
                //usuario no existe
                else
                {
                    var dialog = new MessageDialog(loader.GetString("UserNotExists"));
                    await dialog.ShowAsync();
                }
            }
            catch (Exception e)
            {
                var dialog = new MessageDialog(loader.GetString("ErrorLogin"));
                await dialog.ShowAsync();
            }


            DisableProgressRing();

            return loginOK;
        }

        private void EnableProgressRing()
        {
            progressRing.IsActive = true;
            progressRing.Visibility = Visibility.Visible;
        }

        private void DisableProgressRing()
        {
            progressRing.IsActive = false;
            progressRing.Visibility = Visibility.Collapsed;
        }

        private bool CheckUserNameIsNotEmpty()
        {
            if (string.IsNullOrEmpty(this.userTxt.Text))
            {
                var dialog = new MessageDialog(loader.GetString("UserNameEmpty"));
                dialog.ShowAsync();
                return false;
            }

            return true;
        }

        private bool CheckPasswordIsNotEmpty()
        {
            if (string.IsNullOrEmpty(this.passTxt.Password))
            {
                var dialog = new MessageDialog(loader.GetString("PasswordEmpty"));
                dialog.ShowAsync();
                return false;
            }

            return true;
        }

        private void txtAlta_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(UserRegistration));
        }
    }
}
