﻿using e_teaching.Model;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage.Streams;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

// La plantilla de elemento Página en blanco está documentada en http://go.microsoft.com/fwlink/?LinkId=234238

namespace e_teaching
{
    /// <summary>
    /// Una página vacía que se puede usar de forma independiente o a la que se puede navegar dentro de un objeto Frame.
    /// </summary>
    public sealed partial class ManageAcademicRecord : Page
    {
        ObservableCollection<Profile> ListaAlumnosSourceCollection;
        ObservableCollection<AcademicRecord> ListaCalificacionesSourceCollection;
        ObservableCollection<Subject> ListaAsignaturasSourceCollection;
        Profile profileLogged = new Profile();
        Subject subjectSelected = new Subject();
        Profile studentSelectedRecord = new Profile();
        ResourceLoader loader = new ResourceLoader();

        public ManageAcademicRecord()
        {
            this.InitializeComponent();
            GetInfo();
        }

        private async void GetInfo()
        {
            //comprobar si perfil tiene info
            var prof = await App.MobileService.GetTable<Profile>().Where(p => p.ID == MainPage.UserLogged.ID).ToListAsync();
            if (prof.Any())
            {
                FillProfileInfo(prof.FirstOrDefault());

                //cargar alumnos
                await FillStudentsList();
            }
            else
            {
                this.gridInfoPerfil.Visibility = Visibility.Collapsed;
            }

        }

        private void FillProfileInfo(Profile prof)
        {
            var profile = prof;
            profileLogged = prof;

            this.gridInfoPerfil.Visibility = Visibility.Visible;

            this.txtNombre.Text = profile.Name;
            this.txtApellidos.Text = profile.Surname;

            BitmapImage bitmapImage = new BitmapImage();
            bitmapImage.DecodePixelWidth = 600;
            bitmapImage = ConvertToImage(profile.Image);

            this.ImageProfile.Source = bitmapImage;
        }

        private async Task FillStudentsList()
        {
            EnableProgressRing();
            //estudiantes asociados a profesor           

            var listProfessorStudents = await App.MobileService.GetTable<ProfessorStudent>().Where(p => p.IDProfessor == MainPage.UserLogged.ID).Select(p => p.IDStudent).ToListAsync();

            if (listProfessorStudents.Any())
            {
                var studentsList = await App.MobileService.GetTable<Profile>().Where(s => listProfessorStudents.Contains(s.ID)).ToListAsync();

                this.ListaAlumnosSourceCollection = new ObservableCollection<Profile>(studentsList);

                this.listViewAlumnos.ItemsSource = ListaAlumnosSourceCollection;

                this.listViewAlumnos.Visibility = Visibility.Visible;
            }
            else
            {
                this.listViewAlumnos.Visibility = Visibility.Collapsed;
            }

            DisableProgressRing();
        }

        public BitmapImage ConvertToImage(byte[] value)
        {
            byte[] rawImage = value;

            using (InMemoryRandomAccessStream ms = new InMemoryRandomAccessStream())
            {
                using (DataWriter writer = new DataWriter(ms.GetOutputStreamAt(0)))
                {
                    writer.WriteBytes((byte[])rawImage);

                    // The GetResults here forces to wait until the operation completes
                    // (i.e., it is executed synchronously), so this call can block the UI.
                    writer.StoreAsync().GetResults();
                }

                BitmapImage image = new BitmapImage();
                image.SetSource(ms);
                return image;
            }
        }

        private void txtEditarInfo_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(EnterProfileInfo));
        }

        private async Task<bool> CheckStudentSelected()
        {
            if (listViewAlumnos.SelectedItems.Count > 0)
            {
                return true;
            }
            else
            {
                var dialog = new MessageDialog(loader.GetString("ShouldSelectStudentTask"));
                await dialog.ShowAsync();
                return false;
            }
        }

        private async void txtCerrarSesion_Tapped(object sender, TappedRoutedEventArgs e)
        {
            await LogOut();
        }

        private async Task LogOut()
        {
            MessageDialog showDialog = new MessageDialog(loader.GetString("SureLogout"));
            showDialog.Commands.Add(new UICommand(loader.GetString("Yes"))
            {
                Id = 0
            });
            showDialog.Commands.Add(new UICommand(loader.GetString("No"))
            {
                Id = 1
            });
            showDialog.DefaultCommandIndex = 0;
            showDialog.CancelCommandIndex = 1;
            var result = await showDialog.ShowAsync();
            if ((int)result.Id == 0)
            {
                //borrar settings de sesión
                DeleteAutoLogin();

                //navegar a menú de login
                this.Frame.Navigate(typeof(MainPage));
            }
        }

        public void DeleteAutoLogin()
        {
            MainPage.localSettings.Values.Remove("user");
            MainPage.localSettings.Values.Remove("pass");
        }


        private void EnableProgressRing()
        {
            progressRing.IsActive = true;
            progressRing.Visibility = Visibility.Visible;
        }

        private void DisableProgressRing()
        {
            progressRing.IsActive = false;
            progressRing.Visibility = Visibility.Collapsed;
        }

        private void btmHome_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(MenuProfessor));
        }

        private void listViewAlumnos_ItemClick(object sender, ItemClickEventArgs e)
        {
            var studentSelected = e.ClickedItem as Profile;
            studentSelectedRecord = studentSelected;
            OpenAcademicRecord(studentSelected);

        }

        private async void OpenAcademicRecord(Profile student)
        {
            this.gridAlumnos.Visibility = Visibility.Collapsed;
            this.gridExpediente.Visibility = Visibility.Visible;

            this.txtNombreAlumno.Text = student.Name;
            this.txtApellidosAlumno.Text = student.Surname;

            await GetRecords(student.ID);

        }

        private async Task GetRecords(string idStudent)
        {
            EnableProgressRing();

            var recordsList = await App.MobileService.GetTable<AcademicRecord>().Where(a => a.StudentId == idStudent).ToListAsync();

            if (recordsList.Any())
            {
                this.ListaCalificacionesSourceCollection = new ObservableCollection<AcademicRecord>(recordsList);

                this.listViewCalificaciones.ItemsSource = ListaCalificacionesSourceCollection;

                this.listViewCalificaciones.Visibility = Visibility.Visible;
            }
            else
            {
                this.listViewCalificaciones.Visibility = Visibility.Collapsed;
            }

            DisableProgressRing();
        }

        private async Task GetSubjects()
        {
            EnableProgressRing();

            var subjList = await App.MobileService.GetTable<Subject>().Where(a => a.ProfessorId == MainPage.UserLogged.ID).ToListAsync();


            if (subjList.Any())
            {
                this.ListaAsignaturasSourceCollection = new ObservableCollection<Subject>(subjList);

                this.listViewAsignaturas.ItemsSource = ListaAsignaturasSourceCollection;

                this.listViewAsignaturas.Visibility = Visibility.Visible;
            }
            else
            {
                this.listViewAsignaturas.Visibility = Visibility.Collapsed;
            }

            DisableProgressRing();

        }

        private async void btnEliminarCalif_Tapped(object sender, TappedRoutedEventArgs e)
        {
            var markToDelete = this.listViewCalificaciones.SelectedItem as AcademicRecord;

            await App.MobileService.GetTable<AcademicRecord>().DeleteAsync(markToDelete);

            await GetRecords(studentSelectedRecord.ID);

            this.btnEliminarCalif.Visibility = Visibility.Collapsed;

        }

        private async void btnAñadirCalif_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.btnEliminarCalif.Visibility = Visibility.Collapsed;
            this.gridExpediente.Visibility = Visibility.Collapsed;
            this.gridAñadirCalificacion.Visibility = Visibility.Visible;
            await GetSubjects();
        }

        private async void btnAñadirAsignatura_Tapped(object sender, TappedRoutedEventArgs e)
        {
            var exito = await AddSubject(this.txtNombreAsignatura.Text);
            if (exito)
            {
                //cerrar añadir asignatura 
                this.gridAñadirAsignatura.Visibility = Visibility.Collapsed;
                this.txtNombreAsignatura.Text = string.Empty;

                await GetSubjects();
            }
        }

        private async Task<bool> AddSubject(string subjectName)
        {
            var subject = new Subject
            {
                ProfessorId = MainPage.UserLogged.ID,
                SubjectName = subjectName
            };

            var subjectList = await App.MobileService.GetTable<Subject>().Where(u => u.SubjectName == subjectName && u.ProfessorId == MainPage.UserLogged.ID).ToListAsync();

            if (!subjectList.Any())
            {
                await App.MobileService.GetTable<Subject>().InsertAsync(subject);
                return true;
            }
            else
            {
                //ya existe
                var dialog = new MessageDialog(loader.GetString("SubjectExists"));
                await dialog.ShowAsync();
                return false;
            }
        }

        private void listViewCalificaciones_ItemClick(object sender, ItemClickEventArgs e)
        {
            this.btnEliminarCalif.Visibility = Visibility.Visible;
        }

        private void listViewAsignaturas_ItemClick(object sender, ItemClickEventArgs e)
        {
            this.gridAñadirCalificacionNum.Visibility = Visibility.Visible;
            subjectSelected = e.ClickedItem as Subject;
        }

        private void txtAñadirAsignatura_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.gridAñadirAsignatura.Visibility = Visibility.Visible;
        }

        private async void btnAñadirCalificacionNum_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (await AddMark(subjectSelected) == true)
            {
                this.gridAñadirCalificacionNum.Visibility = Visibility.Collapsed;
                this.txtPuntuacion.Text = string.Empty;
                this.gridAñadirCalificacion.Visibility = Visibility.Collapsed;

                await GetRecords(studentSelectedRecord.ID);

                this.gridExpediente.Visibility = Visibility.Visible;
            }
        }

        private async Task<bool> AddMark(Subject subject)
        {
            var style = NumberStyles.Number;
            var culture = CultureInfo.CurrentUICulture;
            decimal number;
            var puntuacion = this.txtPuntuacion.Text.Replace(".", ",");

            if (decimal.TryParse(puntuacion, style, culture, out number))
            {
                var calif = new AcademicRecord
                {
                    Mark = number,
                    StudentId = studentSelectedRecord.ID,
                    SubjectId = subject.ID,
                    Subject = subject.SubjectName
                };
                try
                {
                    EnableProgressRing();

                    await App.MobileService.GetTable<AcademicRecord>().InsertAsync(calif);

                }
                catch (Exception e)
                {
                    return false;
                    var dialog = new MessageDialog(loader.GetString("ErrorAddingMark"));
                    await dialog.ShowAsync();
                }

                DisableProgressRing();

                var dialog2 = new MessageDialog(loader.GetString("MarkAdded"));
                await dialog2.ShowAsync();
                return true;
            }
            else
            {
                var dialog3 = new MessageDialog(loader.GetString("MarkFormatError"));
                await dialog3.ShowAsync();
                return false;
            }


        }

        private void btmManageAcademicRecord_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(ManageAcademicRecord));
        }


    }
}
