﻿using e_teaching.Model;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage.Streams;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

// La plantilla de elemento Página en blanco está documentada en http://go.microsoft.com/fwlink/?LinkId=234238

namespace e_teaching
{
    /// <summary>
    /// Una página vacía que se puede usar de forma independiente o a la que se puede navegar dentro de un objeto Frame.
    /// </summary>
    public sealed partial class MenuProfessor : Page
    {
        ObservableCollection<Profile> ListaAlumnosSourceCollection;
        ObservableCollection<Profile> ListaAlumnosNoRelacionadosSourceCollection;
        ObservableCollection<Message> ListaMensajesSourceCollection;
        Profile profileLogged = new Profile();
        ResourceLoader loader = new ResourceLoader();

        public MenuProfessor()
        {
            this.InitializeComponent();
            GetInfo();
           
        }

        

        private async void GetInfo()
        {
            //comprobar si perfil tiene info
            var prof = await App.MobileService.GetTable<Profile>().Where(p => p.ID == MainPage.UserLogged.ID).ToListAsync();
            if (prof.Any())
            {
                FillProfileInfo(prof.FirstOrDefault());

                //cargar alumnos
                await FillStudentsList();

                //cargar mensajes
                await FillMessagesList();

            }
            else
            {
                this.gridSinInfo.Visibility = Visibility.Visible;
                this.gridInfoPerfil.Visibility = Visibility.Collapsed;
            }

        }

        private void FillProfileInfo(Profile prof)
        {
            var profile = prof;
            profileLogged = prof;

            this.gridSinInfo.Visibility = Visibility.Collapsed;
            this.gridInfoPerfil.Visibility = Visibility.Visible;

            this.txtNombre.Text = profile.Name;
            this.txtApellidos.Text = profile.Surname;

            BitmapImage bitmapImage = new BitmapImage();
            bitmapImage.DecodePixelWidth = 600;
            bitmapImage = ConvertToImage(profile.Image);

            this.ImageProfile.Source = bitmapImage;
        }

        private async Task FillStudentsList()
        {
            EnableProgressRing();
            //estudiantes asociados a profesor           

            var listProfessorStudents = await App.MobileService.GetTable<ProfessorStudent>().Where(p => p.IDProfessor == MainPage.UserLogged.ID).Select(p => p.IDStudent).ToListAsync();

            if (listProfessorStudents.Any())
            {
                var studentsList = await App.MobileService.GetTable<Profile>().Where(s => listProfessorStudents.Contains(s.ID)).ToListAsync();

                this.ListaAlumnosSourceCollection = new ObservableCollection<Profile>(studentsList);

                this.listViewAlumnos.ItemsSource = ListaAlumnosSourceCollection;

                this.listViewAlumnos.Visibility = Visibility.Visible;
            }   
            else
            {
                this.listViewAlumnos.Visibility = Visibility.Collapsed;
            }

            DisableProgressRing();
        }

        private async Task FillMessagesList()
        {
            EnableProgressRing();
            //mensajes           

            var listMessages = await App.MobileService.GetTable<Message>().Where(p => p.DestinationId == MainPage.UserLogged.ID).ToListAsync();

            if (listMessages.Any())
            {
                // var studentsList = await App.MobileService.GetTable<Profile>().Where(s => listMessages.Contains(s.)).ToListAsync();

                this.txtNumMensajes.Text = listMessages.Count.ToString();

                this.ListaMensajesSourceCollection = new ObservableCollection<Message>(listMessages);

                this.listViewMensajes.ItemsSource = ListaMensajesSourceCollection.OrderByDescending(m => m.Date);

                this.listViewMensajes.Visibility = Visibility.Visible;
            }
            else
            {
                this.txtNumMensajes.Text = loader.GetString("NoMessages");
                this.listViewMensajes.Visibility = Visibility.Collapsed;
            }

            DisableProgressRing();

        }

        private void gridSinInfo_Tapped(object sender, TappedRoutedEventArgs e)
        {
            //navegar a rellenar información de perfil
            this.Frame.Navigate(typeof(EnterProfileInfo));
        }

        public BitmapImage ConvertToImage(byte[] value)
        {
            byte[] rawImage = value;

            using (InMemoryRandomAccessStream ms = new InMemoryRandomAccessStream())
            {
                using (DataWriter writer = new DataWriter(ms.GetOutputStreamAt(0)))
                {
                    writer.WriteBytes((byte[])rawImage);

                    // The GetResults here forces to wait until the operation completes
                    // (i.e., it is executed synchronously), so this call can block the UI.
                    writer.StoreAsync().GetResults();
                }

                BitmapImage image = new BitmapImage();
                image.SetSource(ms);
                return image;
            }
        }

        private void txtEditarInfo_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(EnterProfileInfo));
        }

        private async void txtVincularAlumno_Tapped(object sender, TappedRoutedEventArgs e)
        {
            
           var result = await GetUnrelatedStudents();

            if (result)
            {
                this.gridVincularAlumno.Visibility = Visibility.Visible;
                this.gridBuzonMensajes.Visibility = Visibility.Collapsed;
            }
            else
            {
                var dialog = new MessageDialog(loader.GetString("NoStudentsToLink"));
                await dialog.ShowAsync();
            }
        }

        private async Task<bool> GetUnrelatedStudents()
        {
            EnableProgressRing();

            //estudiantes asociados a profesor
            var listStudentsRelated = await App.MobileService.GetTable<ProfessorStudent>().Where(s => s.IDProfessor == profileLogged.ID).Select(p => p.IDStudent).ToListAsync();

            var listStudents = await App.MobileService.GetTable<User>().Where(u => u.Role == 2).Select(p => p.ID).ToListAsync();

            var unrelatedStudentsList = new List<Profile>();
            //estudiantes no relacionados con profesor
            if (listStudentsRelated.Any())
            {
               unrelatedStudentsList = await App.MobileService.GetTable<Profile>().Where(s => !listStudentsRelated.Contains(s.ID) && listStudents.Contains(s.ID)).ToListAsync();
            }
            //else
            //{
            //    unrelatedStudentsList = await App.MobileService.GetTable<Profile>().Where(s => listStudents.Contains(s.ID)).ToListAsync();
            //}

            if (!unrelatedStudentsList.Any())
            {
                DisableProgressRing();
                return false;
            }

            this.ListaAlumnosNoRelacionadosSourceCollection = new ObservableCollection<Profile>(unrelatedStudentsList);

            this.listViewAlumnosDesvinculados.ItemsSource = ListaAlumnosNoRelacionadosSourceCollection;

            DisableProgressRing();

            return true;
        }

        private async void btnVincularAlumno_Tapped(object sender, TappedRoutedEventArgs e)
        {
            EnableProgressRing(); 

            if (listViewAlumnosDesvinculados.SelectedItems.Count > 0)
            {
                try
                { //vincular alumnos seleccionados
                    foreach (Profile alumno in listViewAlumnosDesvinculados.SelectedItems)
                    {
                        ProfessorStudent ps = new ProfessorStudent();
                        ps.IDProfessor = MainPage.UserLogged.ID;
                        ps.IDStudent = alumno.ID;

                        await App.MobileService.GetTable<ProfessorStudent>().InsertAsync(ps);
                    }

                }
                catch (Exception ex)
                {

                }
            }
                //cerrar vista vinculación
                this.gridVincularAlumno.Visibility = Visibility.Collapsed;

                //cargar lista alumnos vinculados
                await FillStudentsList();

            this.gridBuzonMensajes.Visibility = Visibility.Visible;

            DisableProgressRing();

        }

        private void listViewAlumnos_ItemClick(object sender, ItemClickEventArgs e)
        {
            //activar opcion enviar mensaje
                this.txtEnviarMensaje.Visibility = Visibility.Visible;           
        }

        private async void txtEnviarMensaje_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.txtMensaje.Text = string.Empty;
            this.txtAsunto.Text = string.Empty;

            await CheckStudentSelected();            
        }

        private void btnEnviarMensaje_Tapped(object sender, TappedRoutedEventArgs e)
        {
            if (checkBoxResponsables.IsChecked == true)
            {
                SendMessageToParents();
            }
            else
            {
                SendMessageToStudents();
            }           
        }



        private async Task<bool> CheckStudentSelected()
        {
            if (listViewAlumnos.SelectedItems.Count > 0)
            {   
                this.gridBuzonMensajes.Visibility = Visibility.Collapsed;
                this.gridEnviarMensaje.Visibility = Visibility.Visible;

                return true;
            }
            else
            {               
                var dialog = new MessageDialog(loader.GetString("ShouldSelectStudent"));
                await dialog.ShowAsync();
                return false;
            }
        }

        private async void SendMessageToStudents()
        {
            if (await CheckMessageIsNotEmpty() == true && await CheckStudentSelected() == true)
            {
                try
                {
                    EnableProgressRing();
                    foreach (Profile student in listViewAlumnos.SelectedItems)
                        {
                            Message message = new Message();
                            message.Date = DateTime.Now;
                            message.OriginId = MainPage.UserLogged.ID;
                            message.Name = profileLogged.Name;
                            message.Surname = profileLogged.Surname;
                            message.DestinationId = student.ID;
                            message.Content = this.txtMensaje.Text;
                            message.Subject = string.IsNullOrEmpty(this.txtAsunto.Text) ? "Sin Asunto" : this.txtAsunto.Text;

                            //enviar mensaje
                            await App.MobileService.GetTable<Message>().InsertAsync(message);
                        }
                   
                    
                }catch(Exception e)
                {
                    var dialog = new MessageDialog(loader.GetString("ErrorSendingMessage"));
                    await dialog.ShowAsync();
                }

                DisableProgressRing();

                var dialog2 = new MessageDialog(loader.GetString("MessageSent"));
                await dialog2.ShowAsync();

                this.gridEnviarMensaje.Visibility = Visibility.Collapsed;
                this.gridBuzonMensajes.Visibility = Visibility.Visible;

            }

        }

        private async void SendMessageToParents()
        {
            if (await CheckMessageIsNotEmpty() == true && await CheckStudentSelected() == true)
            {
                try
                {
                    EnableProgressRing();
                    foreach (Profile student in listViewAlumnos.SelectedItems)
                    {
                        var linkedParent = await App.MobileService.GetTable<ParentStudent>().Where(l => l.IDStudent == student.ID).ToListAsync();
                        if (linkedParent.Any())
                        {
                            Message message = new Message();
                            message.Date = DateTime.Now;
                            message.OriginId = MainPage.UserLogged.ID;
                            message.Name = profileLogged.Name;
                            message.Surname = profileLogged.Surname;
                            message.DestinationId = linkedParent.FirstOrDefault().IDParent;
                            message.Content = this.txtMensaje.Text;
                            message.Subject = string.IsNullOrEmpty(this.txtAsunto.Text) ? "Sin Asunto" : this.txtAsunto.Text;

                            //enviar mensaje
                            await App.MobileService.GetTable<Message>().InsertAsync(message);

                            var dialog2 = new MessageDialog(loader.GetString("MessageSent"), string.Concat(student.Name, " ", student.Surname));
                            await dialog2.ShowAsync();
                        }
                        else
                        {                            
                            var dialog = new MessageDialog(loader.GetString("StudentNotLinked"), string.Concat(student.Name, " ", student.Surname));
                            await dialog.ShowAsync();
                        }                        
                    }
                }
                catch (Exception e)
                {
                    var dialog = new MessageDialog(loader.GetString("ErrorSendingMessage"));
                    await dialog.ShowAsync();
                }

                DisableProgressRing();                

                this.gridEnviarMensaje.Visibility = Visibility.Collapsed;
                this.gridBuzonMensajes.Visibility = Visibility.Visible;

            }

        }

        private async Task<bool> CheckMessageIsNotEmpty()
        {
            if (string.IsNullOrEmpty(this.txtMensaje.Text))
            {
                var dialog = new MessageDialog(loader.GetString("MessageIsEmpty"));
                await dialog.ShowAsync();
                return false;
            }

            return true;
        }

        private async Task<bool> CheckMessageRIsNotEmpty()
        {
            if (string.IsNullOrEmpty(this.txtMensajeR.Text))
            {
                var dialog = new MessageDialog(loader.GetString("MessageIsEmpty"));
                await dialog.ShowAsync();
                return false;
            }

            return true;
        }

        private void listViewMensajes_ItemClick(object sender, ItemClickEventArgs e)
        {
            var mensaje = (Message)e.ClickedItem;
            OpenMessage(mensaje);
        }

        private void OpenMessage(Message mensaje)
        {
            //ocultar lista mensajes
            this.gridBuzonMensajes.Visibility = Visibility.Collapsed;

            //cargar mensaje
            this.gridLeerMensaje.DataContext = mensaje;

            //abrir grid leer mensaje
            this.gridLeerMensaje.Visibility = Visibility.Visible;
        }

        private async void txtCerrarSesion_Tapped(object sender, TappedRoutedEventArgs e)
        {
           await LogOut();
        }

        private async Task LogOut()
        {
            MessageDialog showDialog = new MessageDialog(loader.GetString("SureLogout"));
            showDialog.Commands.Add(new UICommand(loader.GetString("Yes"))
            {
                Id = 0
            });
            showDialog.Commands.Add(new UICommand(loader.GetString("No"))
            {
                Id = 1
            });
            showDialog.DefaultCommandIndex = 0;
            showDialog.CancelCommandIndex = 1;
            var result = await showDialog.ShowAsync();
            if ((int)result.Id == 0)
            {
                //borrar settings de sesión
                DeleteAutoLogin();

                //navegar a menú de login
                this.Frame.Navigate(typeof(MainPage));
            }
        }

        private void btnCerrar_Tapped(object sender, TappedRoutedEventArgs e)
        {
            CloseMessage();
        }

        private void CloseMessage()
        {
            this.gridLeerMensaje.Visibility = Visibility.Collapsed;
            this.gridBuzonMensajes.Visibility = Visibility.Visible;
        }

        public void DeleteAutoLogin()
        {
            MainPage.localSettings.Values.Remove("user");
            MainPage.localSettings.Values.Remove("pass");
        }

        private async Task DeleteMessage(Message message)
        {
            await App.MobileService.GetTable<Message>().DeleteAsync(message);
        }

        private async void btnEliminar_Tapped(object sender, TappedRoutedEventArgs e)
        {
            var message = (Message)this.gridLeerMensaje.DataContext;

            await DeleteMessage(message);

            //cargar mensajes
            await FillMessagesList();
            CloseMessage();
        }

        private void btnResponder_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.gridLeerMensaje.Visibility = Visibility.Collapsed;
            
            this.gridResponderMensaje.Visibility = Visibility.Visible;
        }

        private async void ReplyMessage(Message messageR)
        {
            if (await CheckMessageRIsNotEmpty() == true)
            {
                try
                {
                        Message message = new Message();
                        message.Date = DateTime.Now;
                        message.OriginId = MainPage.UserLogged.ID;
                        message.Name = profileLogged.Name;
                        message.Surname = profileLogged.Surname;
                        message.DestinationId = messageR.OriginId;
                        message.Content = this.txtMensajeR.Text;
                        message.Subject = string.IsNullOrEmpty(this.txtAsuntoR.Text) ? "Sin Asunto" : this.txtAsuntoR.Text;

                        //enviar mensaje
                        await App.MobileService.GetTable<Message>().InsertAsync(message);        
                }
                catch (Exception e)
                {
                    var dialog = new MessageDialog(loader.GetString("ErrorSendingMessage"));
                    await dialog.ShowAsync();
                }

                var dialog2 = new MessageDialog(loader.GetString("MessageSent"));
                await dialog2.ShowAsync();

                this.gridResponderMensaje.Visibility = Visibility.Collapsed;
                this.gridBuzonMensajes.Visibility = Visibility.Visible;

            }

        }

        private void btnEnviarMensajeR_Tapped(object sender, TappedRoutedEventArgs e)
        {
            ReplyMessage((Message)this.gridLeerMensaje.DataContext);
        }

        private void EnableProgressRing()
        {
            progressRing.IsActive = true;
            progressRing.Visibility = Visibility.Visible;
        }

        private void DisableProgressRing()
        {
            progressRing.IsActive = false;
            progressRing.Visibility = Visibility.Collapsed;
        }

        private void checkBoxResponsables_Checked(object sender, RoutedEventArgs e)
        {
            this.txtEnviarMensajeAAlumnosResponsables.Text = loader.GetString("MessateToParents");
        }

        private void checkBoxResponsables_Unchecked(object sender, RoutedEventArgs e)
        {
            this.txtEnviarMensajeAAlumnosResponsables.Text = loader.GetString("MessageToStudents");
        }

        private void btmAddTask_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(AddTask));
        }

        private void btmManageAcademicRecord_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(ManageAcademicRecord));
        }
    }
}
