﻿using e_teaching.Model;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage.Streams;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

// La plantilla de elemento Página en blanco está documentada en http://go.microsoft.com/fwlink/?LinkId=234238

namespace e_teaching
{
    /// <summary>
    /// Una página vacía que se puede usar de forma independiente o a la que se puede navegar dentro de un objeto Frame.
    /// </summary>
    public sealed partial class MenuStudent : Page
    {
        ObservableCollection<Message> ListaMensajesSourceCollection;
        ObservableCollection<Profile> ListaProfesoresSourceCollection;
        Profile profileLogged = new Profile();
        ResourceLoader loader = new ResourceLoader();

        public MenuStudent()
        {
            this.InitializeComponent();
            GetInfo();           
        }



        private async void GetInfo()
        {
            //comprobar si perfil tiene info
            var prof = await App.MobileService.GetTable<Profile>().Where(p => p.ID == MainPage.UserLogged.ID).ToListAsync();
            if (prof.Any())
            {
                FillProfileInfo(prof.FirstOrDefault());
                await GetLinkedParent();

                //cargar alumnos
                await FillProfessorsList();

                //cargar mensajes
                await FillMessagesList();

            }
            else
            {
                this.gridSinInfo.Visibility = Visibility.Visible;
                this.gridInfoPerfil.Visibility = Visibility.Collapsed;
                this.gridBuzonMensajes.Visibility = Visibility.Collapsed;
                this.gridProfesores.Visibility = Visibility.Collapsed;
            }
        }

        private async Task GetLinkedParent()
        {
            var linkedList = await App.MobileService.GetTable<ParentStudent>().Where(p => p.IDStudent == MainPage.UserLogged.ID).ToListAsync();
            if (!linkedList.Any())
            {
                this.txtAsociarPadre.Visibility = Visibility.Visible;
            }
            else
            {
                this.txtAsociarPadre.Visibility = Visibility.Collapsed;
            }
        }

        private async Task LinkParent(string code)
        {
            if (CheckLinkCodeIsNotEmpty() == true)
            {
                var link = await App.MobileService.GetTable<ParentStudent>().Where(c => c.IDLink == Convert.ToInt32(code) && c.IDStudent == "").ToListAsync();
                if (link.Any())
                {
                    //asociar
                    var parentStudent = new ParentStudent()
                    {
                        IDStudent = profileLogged.ID,
                        IDLink = link.FirstOrDefault().IDLink,
                        ID = link.FirstOrDefault().ID,
                        IDParent = link.FirstOrDefault().IDParent
                    };

                    await App.MobileService.GetTable<ParentStudent>().UpdateAsync(parentStudent);

                    var dialog = new MessageDialog(loader.GetString("LinkedOk"));
                    await dialog.ShowAsync();

                    this.gridAsociacion.Visibility = Visibility.Collapsed;
                    this.txtAsociarPadre.Visibility = Visibility.Collapsed;
                    ShowAll();

                }
                else
                {
                    var dialog = new MessageDialog(loader.GetString("LinkCodeNotCorrect"));
                    await dialog.ShowAsync();
                }
            }
        }

        private void HideAll()
        {
            this.gridBuzonMensajes.Visibility = Visibility.Collapsed;
            this.gridProfesores.Visibility = Visibility.Collapsed;
        }

        private void ShowAll()
        {
            this.gridBuzonMensajes.Visibility = Visibility.Visible;
            this.gridProfesores.Visibility = Visibility.Visible;
        }

        private async void btnAsociar_Tapped(object sender, TappedRoutedEventArgs e)
        {
            await LinkParent(this.txtCodigoAsociacion.Text);
        }

        private bool CheckLinkCodeIsNotEmpty()
        {
            if (string.IsNullOrEmpty(this.txtCodigoAsociacion.Text))
            {
                var dialog = new MessageDialog(loader.GetString("LinkCodeIsEmpty"));
                dialog.ShowAsync();
                return false;
            }

            return true;
        }

        private void FillProfileInfo(Profile prof)
        {
            var profile = prof;
            profileLogged = profile;

            this.gridSinInfo.Visibility = Visibility.Collapsed;
            this.gridInfoPerfil.Visibility = Visibility.Visible;

            this.txtNombre.Text = profile.Name;
            this.txtApellidos.Text = profile.Surname;

            BitmapImage bitmapImage = new BitmapImage();
            bitmapImage.DecodePixelWidth = 600;
            bitmapImage = ConvertToImage(profile.Image);

            this.ImageProfile.Source = bitmapImage;
        }

        private async Task FillProfessorsList()
        {
            //profesores asociados a estudiante           

            var listProfessorStudents = await App.MobileService.GetTable<ProfessorStudent>().Where(p => p.IDStudent == MainPage.UserLogged.ID).Select(p => p.IDProfessor).ToListAsync();

            if (listProfessorStudents.Any())
            {
                this.gridProfesores.Visibility = Visibility.Visible;
                var profesorsList = await App.MobileService.GetTable<Profile>().Where(s => listProfessorStudents.Contains(s.ID)).ToListAsync();

                this.ListaProfesoresSourceCollection = new ObservableCollection<Profile>(profesorsList);

                this.listViewProfesores.ItemsSource = ListaProfesoresSourceCollection;

                this.listViewProfesores.Visibility = Visibility.Visible;
            }
            else
            {
                this.gridProfesores.Visibility = Visibility.Collapsed;
                this.listViewProfesores.Visibility = Visibility.Collapsed;
            }

        }

        private async Task FillMessagesList()
        {
            //mensajes           

            var listMessages = await App.MobileService.GetTable<Message>().Where(p => p.DestinationId == profileLogged.ID).ToListAsync();
           
            if (listMessages.Any())
            {

                this.txtNumMensajes.Text = listMessages.Count.ToString();

                this.ListaMensajesSourceCollection = new ObservableCollection<Message>(listMessages);

                this.listViewMensajes.ItemsSource = ListaMensajesSourceCollection.OrderByDescending(m => m.Date);

                this.listViewMensajes.Visibility = Visibility.Visible;
            }
            else
            {
                this.txtNumMensajes.Text = loader.GetString("NoMessages");
                this.listViewMensajes.Visibility = Visibility.Collapsed;
            }

        }

        private void gridSinInfo_Tapped(object sender, TappedRoutedEventArgs e)
        {
            //navegar a rellenar información de perfil
            this.Frame.Navigate(typeof(EnterProfileInfo));
        }

        public BitmapImage ConvertToImage(byte[] value)
        {
            byte[] rawImage = value;

            using (InMemoryRandomAccessStream ms = new InMemoryRandomAccessStream())
            {
                using (DataWriter writer = new DataWriter(ms.GetOutputStreamAt(0)))
                {
                    writer.WriteBytes((byte[])rawImage);

                    // The GetResults here forces to wait until the operation completes
                    // (i.e., it is executed synchronously), so this call can block the UI.
                    writer.StoreAsync().GetResults();
                }

                BitmapImage image = new BitmapImage();
                image.SetSource(ms);
                return image;
            }
        }

        private void txtEditarInfo_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(EnterProfileInfo));
        }

        private void listViewMensajes_ItemClick(object sender, ItemClickEventArgs e)
        {
            var mensaje = (Message)e.ClickedItem;
            OpenMessage(mensaje);
        }

        private void listViewProfesores_ItemClick(object sender, ItemClickEventArgs e)
        {
            //activar opcion enviar mensaje
            this.txtEnviarMensaje.Visibility = Visibility.Visible;
        }

        private void txtEnviarMensaje_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.txtMensaje.Text = string.Empty;
            this.gridBuzonMensajes.Visibility = Visibility.Collapsed;
            this.gridEnviarMensaje.Visibility = Visibility.Visible;
        }

        private void btnEnviarMensaje_Tapped(object sender, TappedRoutedEventArgs e)
        {
            SendMessageToProfessors();
        }

        private async void SendMessageToProfessors()
        {
            if (await CheckMessageIsNotEmpty() == true)
            {
                try
                {
                    foreach (Profile professor in listViewProfesores.SelectedItems)
                    {
                        Message message = new Message();
                        message.Date = DateTime.Now;
                        message.OriginId = MainPage.UserLogged.ID;
                        message.Name = profileLogged.Name;
                        message.Surname = profileLogged.Surname;
                        message.DestinationId = professor.ID;
                        message.Content = this.txtMensaje.Text;
                        message.Subject = string.IsNullOrEmpty(this.txtAsunto.Text) ? "Sin Asunto" : this.txtAsunto.Text;

                        //enviar mensaje
                        await App.MobileService.GetTable<Message>().InsertAsync(message);
                    }
                }
                catch (Exception e)
                {
                    var dialog = new MessageDialog(loader.GetString("ErrorSendingMessage"));
                    await dialog.ShowAsync();
                }

                var dialog2 = new MessageDialog(loader.GetString("MessageSent"));
                await dialog2.ShowAsync();

                this.gridEnviarMensaje.Visibility = Visibility.Collapsed;
                this.gridBuzonMensajes.Visibility = Visibility.Visible;

            }
        }

        private async Task<bool> CheckMessageIsNotEmpty()
        {
            if (string.IsNullOrEmpty(this.txtMensaje.Text))
            {
                var dialog = new MessageDialog(loader.GetString("MessageIsEmpty"));
                await dialog.ShowAsync();
                return false;
            }

            return true;
        }

        private async Task<bool> CheckMessageRIsNotEmpty()
        {
            if (string.IsNullOrEmpty(this.txtMensajeR.Text))
            {
                var dialog = new MessageDialog(loader.GetString("MessageIsEmpty"));
                await dialog.ShowAsync();
                return false;
            }

            return true;
        }

        private void OpenMessage(Message mensaje)
        {
            //ocultar lista mensajes
            this.gridBuzonMensajes.Visibility = Visibility.Collapsed;

            //cargar mensaje
            this.gridLeerMensaje.DataContext = mensaje;

            //abrir grid leer mensaje
            this.gridLeerMensaje.Visibility = Visibility.Visible;
        }

        private void btnCerrar_Tapped(object sender, TappedRoutedEventArgs e)
        {
            CloseMessage();
        }

        private async void txtCerrarSesion_Tapped(object sender, TappedRoutedEventArgs e)
        {
            await LogOut();
        }

        private async Task LogOut()
        {
            MessageDialog showDialog = new MessageDialog(loader.GetString("SureLogout"));
            showDialog.Commands.Add(new UICommand(loader.GetString("Yes"))
            {
                Id = 0
            });
            showDialog.Commands.Add(new UICommand(loader.GetString("No"))
            {
                Id = 1
            });
            showDialog.DefaultCommandIndex = 0;
            showDialog.CancelCommandIndex = 1;
            var result = await showDialog.ShowAsync();
            if ((int)result.Id == 0)
            {
                //borrar settings de sesión
                DeleteAutoLogin();

                //navegar a menú de login
                this.Frame.Navigate(typeof(MainPage));
            }
        }

        public void DeleteAutoLogin()
        {
            MainPage.localSettings.Values.Remove("user");
            MainPage.localSettings.Values.Remove("pass");
        }

        private void btnEnviarMensajeR_Tapped(object sender, TappedRoutedEventArgs e)
        {
            ReplyMessage((Message)this.gridLeerMensaje.DataContext);
        }

        private void btnResponder_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.gridLeerMensaje.Visibility = Visibility.Collapsed;

            this.gridResponderMensaje.Visibility = Visibility.Visible;
        }

        private void CloseMessage()
        {
            this.gridLeerMensaje.Visibility = Visibility.Collapsed;
            this.gridBuzonMensajes.Visibility = Visibility.Visible;
        }

        private async void ReplyMessage(Message messageR)
        {
            if (await CheckMessageRIsNotEmpty() == true)
            {
                try
                {
                    Message message = new Message();
                    message.Date = DateTime.Now;
                    message.OriginId = MainPage.UserLogged.ID;
                    message.Name = profileLogged.Name;
                    message.Surname = profileLogged.Surname;
                    message.DestinationId = messageR.OriginId;
                    message.Content = this.txtMensajeR.Text;
                    message.Subject = string.IsNullOrEmpty(this.txtAsuntoR.Text) ? "Sin Asunto" : this.txtAsuntoR.Text;

                    //enviar mensaje
                    await App.MobileService.GetTable<Message>().InsertAsync(message);
                }
                catch (Exception e)
                {
                    var dialog = new MessageDialog(loader.GetString("ErrorSendingMessage"));
                    await dialog.ShowAsync();
                }

                var dialog2 = new MessageDialog(loader.GetString("MessageSent"));
                await dialog2.ShowAsync();

                this.gridResponderMensaje.Visibility = Visibility.Collapsed;
                this.gridBuzonMensajes.Visibility = Visibility.Visible;

            }

        }

        private async Task DeleteMessage(Message message)
        {
            await App.MobileService.GetTable<Message>().DeleteAsync(message);
        }

        private async void btnEliminar_Tapped(object sender, TappedRoutedEventArgs e)
        {
            var message = (Message)this.gridLeerMensaje.DataContext;

            await DeleteMessage(message);

            //cargar mensajes
            await FillMessagesList();
            CloseMessage();
        }

        private void txtAsociarPadre_Tapped(object sender, TappedRoutedEventArgs e)
        {
            HideAll();

            this.gridAsociacion.Visibility = Visibility.Visible;

        }

        private void btmViewTasks_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(ViewTasks));
        }

        private void btmViewAcademicRecord_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(ShowAcademicRecord));
        }
    }
}
