﻿using e_teaching.Model;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage.Streams;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

// La plantilla de elemento Página en blanco está documentada en http://go.microsoft.com/fwlink/?LinkId=234238

namespace e_teaching
{
    /// <summary>
    /// Una página vacía que se puede usar de forma independiente o a la que se puede navegar dentro de un objeto Frame.
    /// </summary>
    public sealed partial class ShowAcademicRecordParent : Page
    {
        ObservableCollection<Profile> ListaAlumnosSourceCollection;
        ObservableCollection<AcademicRecord> ListaCalificacionesSourceCollection;
        ObservableCollection<Subject> ListaAsignaturasSourceCollection;
        Profile profileLogged = new Profile();
        Subject subjectSelected = new Subject();
        Profile studentSelectedRecord = new Profile();
        ResourceLoader loader = new ResourceLoader();

        public ShowAcademicRecordParent()
        {
            this.InitializeComponent();
            GetInfo();
        }

        private async void GetInfo()
        {
            //comprobar si perfil tiene info
            var prof = await App.MobileService.GetTable<Profile>().Where(p => p.ID == MainPage.UserLogged.ID).ToListAsync();
            if (prof.Any())
            {
                FillProfileInfo(prof.FirstOrDefault());

                //cargar expediente
                await GetRecords(MenuParent.IdStudentLinked);
            }
            else
            {
                this.gridInfoPerfil.Visibility = Visibility.Collapsed;
            }
        }

        private void FillProfileInfo(Profile prof)
        {
            var profile = prof;
            profileLogged = prof;

            this.gridInfoPerfil.Visibility = Visibility.Visible;

            this.txtNombre.Text = profile.Name;
            this.txtApellidos.Text = profile.Surname;

            BitmapImage bitmapImage = new BitmapImage();
            bitmapImage.DecodePixelWidth = 600;
            bitmapImage = ConvertToImage(profile.Image);

            this.ImageProfile.Source = bitmapImage;
        }

        public BitmapImage ConvertToImage(byte[] value)
        {
            byte[] rawImage = value;

            using (InMemoryRandomAccessStream ms = new InMemoryRandomAccessStream())
            {
                using (DataWriter writer = new DataWriter(ms.GetOutputStreamAt(0)))
                {
                    writer.WriteBytes((byte[])rawImage);

                    // The GetResults here forces to wait until the operation completes
                    // (i.e., it is executed synchronously), so this call can block the UI.
                    writer.StoreAsync().GetResults();
                }

                BitmapImage image = new BitmapImage();
                image.SetSource(ms);
                return image;
            }
        }

        private void txtEditarInfo_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(EnterProfileInfo));
        }

        private async void txtCerrarSesion_Tapped(object sender, TappedRoutedEventArgs e)
        {
            await LogOut();
        }

        private async Task LogOut()
        {
            MessageDialog showDialog = new MessageDialog(loader.GetString("SureLogout"));
            showDialog.Commands.Add(new UICommand(loader.GetString("Yes"))
            {
                Id = 0
            });
            showDialog.Commands.Add(new UICommand(loader.GetString("No"))
            {
                Id = 1
            });
            showDialog.DefaultCommandIndex = 0;
            showDialog.CancelCommandIndex = 1;
            var result = await showDialog.ShowAsync();
            if ((int)result.Id == 0)
            {
                //borrar settings de sesión
                DeleteAutoLogin();

                //navegar a menú de login
                this.Frame.Navigate(typeof(MainPage));
            }
        }

        public void DeleteAutoLogin()
        {
            MainPage.localSettings.Values.Remove("user");
            MainPage.localSettings.Values.Remove("pass");
        }


        private void EnableProgressRing()
        {
            progressRing.IsActive = true;
            progressRing.Visibility = Visibility.Visible;
        }

        private void DisableProgressRing()
        {
            progressRing.IsActive = false;
            progressRing.Visibility = Visibility.Collapsed;
        }

        private void btmHome_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(MenuParent));
        }

        private async Task GetRecords(string idStudent)
        {
            EnableProgressRing();

            var recordsList = await App.MobileService.GetTable<AcademicRecord>().Where(a => a.StudentId == idStudent).ToListAsync();

            if (recordsList.Any())
            {
                this.ListaCalificacionesSourceCollection = new ObservableCollection<AcademicRecord>(recordsList);

                this.listViewCalificaciones.ItemsSource = ListaCalificacionesSourceCollection;

                this.listViewCalificaciones.Visibility = Visibility.Visible;
            }
            else
            {
                this.txtSinCalificaciones.Visibility = Visibility.Visible;
                this.listViewCalificaciones.Visibility = Visibility.Collapsed;
            }

            DisableProgressRing();
        }

        private void btmViewTasks_Tapped(object sender, TappedRoutedEventArgs e)
        {
            this.Frame.Navigate(typeof(ViewTasks));
        }
    }
}
