﻿using e_teaching.Model;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage.Streams;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

// La plantilla de elemento Página en blanco está documentada en http://go.microsoft.com/fwlink/?LinkId=234238

namespace e_teaching
{
    /// <summary>
    /// Una página vacía que se puede usar de forma independiente o a la que se puede navegar dentro de un objeto Frame.
    /// </summary>
    public sealed partial class UserRegistration : Page
    {

        ResourceLoader loader = new ResourceLoader();

        public UserRegistration()
        {
            this.InitializeComponent();
            FillRoleList();

        }

        private void FillRoleList()
        {
            this.listBoxPerfil.Items.Add("Profesor");
            this.listBoxPerfil.Items.Add("Alumno");
            this.listBoxPerfil.Items.Add("Padre/Madre/Tutor");
        }
        
        private async Task Register()
        {
            EnableProgressRing();

            var user = new User();

            user.UserName = this.txtUser.Text;
            user.Password = this.txtPassword.Password;
            user.Role = this.listBoxPerfil.SelectedIndex + 1;

            //Insertar en bbdd
            var result = await SaveDataInDB(user);
            MainPage.UserLogged = user;

            if (result)
            {
                //parent
                if(user.Role == 3)
                {
                   var ps = RegisterLinkParentStudent(user);

                    var dialog = new MessageDialog(loader.GetString("NoteLinkCode"));
                    await dialog.ShowAsync();

                    var dialog2 = new MessageDialog(ps.Result.ToString());
                    await dialog2.ShowAsync();
                }

                //navegar a pantalla principal
                switch (user.Role)
                {
                    case 1: //profesor
                        Frame.Navigate(typeof(MenuProfessor));
                        break;
                    case 2: //alumno
                        Frame.Navigate(typeof(MenuStudent));
                        break;
                    case 3: //padres
                        Frame.Navigate(typeof(MenuParent));
                        break;
                }
            }

            DisableProgressRing();
            
        }

        private bool CheckPasswordsMatch()
        {
          if(this.txtPassword.Password.Equals(this.txtPassword2.Password))
            {
                return true;
            }
            else
            {
                var dialog = new MessageDialog(loader.GetString("PasswordsNotMatch"));
                dialog.ShowAsync();
                return false;
            }
        }

        private bool CheckUserNameIsNotEmpty()
        {
            if (string.IsNullOrEmpty(this.txtUser.Text))
            {
                var dialog = new MessageDialog(loader.GetString("UserNameEmpty"));
                dialog.ShowAsync();
                return false;
            }

            return true;
        }

        private bool CheckPasswordIsNotEmpty()
        {
            if (string.IsNullOrEmpty(this.txtPassword.Password) && string.IsNullOrEmpty(this.txtPassword2.Password))
            {
                var dialog = new MessageDialog(loader.GetString("PasswordEmpty"));
                dialog.ShowAsync();
                return false;
            }

            return true;
        }

        private bool CheckRoleSelected()
        {
            if(this.listBoxPerfil.SelectedIndex == -1)
            {
                var dialog = new MessageDialog(loader.GetString("RoleNotSelected"));
                dialog.ShowAsync();
                return false;
            }
            else
            {
                return true;
            }
        }

        private async void btnSave_Tapped(object sender, TappedRoutedEventArgs e)
        {

            if (CheckPasswordsMatch() == true && CheckPasswordIsNotEmpty() == true && CheckUserNameIsNotEmpty() == true && CheckRoleSelected() == true)
            {
                Register();
            }               
        }

        private async Task<bool> SaveDataInDB(User user)
        {
            //comprobar existe usuario en bbdd
            var prof = await App.MobileService.GetTable<User>().Where(p => p.UserName == user.UserName).ToListAsync();
            if (prof.Any())
            {
                //ya existe ese usuario
                var dialog = new MessageDialog(loader.GetString("UserNameExists"));
                await dialog.ShowAsync();
                return false;
            }
            else
            {
                //insertar 
               await App.MobileService.GetTable<User>().InsertAsync(user);
                return true;
            }
        }

        private int GetRandonLinkCode()
        {
           return new Random().Next(100, 999999);
        }

        private async Task<int> RegisterLinkParentStudent(User user)
        {

            var parentStudent = new ParentStudent() {
                IDLink = GetRandonLinkCode(),
                IDParent = user.ID,
                IDStudent = string.Empty       
            };
            await App.MobileService.GetTable<ParentStudent>().InsertAsync(parentStudent);

            return parentStudent.IDLink;
        }

        private void EnableProgressRing()
        {
            progressRing.IsActive = true;
            progressRing.Visibility = Visibility.Visible;
        }

        private void DisableProgressRing()
        {
            progressRing.IsActive = false;
            progressRing.Visibility = Visibility.Collapsed;
        }
    }
}
